Type: Package
Package: bayestestR
Title: Understand and Describe Bayesian Models and Posterior
        Distributions
Version: 0.13.2
Authors@R: 
    c(person(given = "Dominique",
             family = "Makowski",
             role = c("aut", "cre"),
             email = "dom.makowski@gmail.com",
             comment = c(ORCID = "0000-0001-5375-9967", Twitter = "@Dom_Makowski")),
      person(given = "Daniel",
             family = "Lüdecke",
             role = "aut",
             email = "d.luedecke@uke.de",
             comment = c(ORCID = "0000-0002-8895-3206", Twitter = "@strengejacke")),
      person(given = "Mattan S.",
             family = "Ben-Shachar",
             role = "aut",
             email = "matanshm@post.bgu.ac.il",
             comment = c(ORCID = "0000-0002-4287-4801", Twitter = "@mattansb")),
      person(given = "Indrajeet",
             family = "Patil",
             role = "aut",
             email = "patilindrajeet.science@gmail.com",
             comment = c(ORCID = "0000-0003-1995-6531", Twitter = "@patilindrajeets")),
      person(given = "Michael D.",
             family = "Wilson",
             role = "aut",
             email = "michael.d.wilson@curtin.edu.au",
             comment = c(ORCID = "0000-0003-4143-7308")),
      person(given = "Brenton M.",
             family = "Wiernik",
             role = "aut",
             email = "brenton@wiernik.org",
             comment = c(ORCID = "0000-0001-9560-6336", Twitter = "@bmwiernik")),
      person(given = "Paul-Christian",
             family = "Bürkner",
             role = "rev",
             email = "paul.buerkner@gmail.com"),
      person(given = "Tristan",
             family = "Mahr",
             role = "rev",
             email = "tristan.mahr@wisc.edu",
             comment = c(ORCID = "0000-0002-8890-5116")),
      person(given = "Henrik",
             family = "Singmann",
             role = "ctb",
             email = "singmann@gmail.com",
             comment = c(ORCID = "0000-0002-4842-3657")),
      person(given = "Quentin F.",
             family = "Gronau",
             role = "ctb",
             comment = c(ORCID = "0000-0001-5510-6943")),
      person(given = "Sam",
             family = "Crawley",
             role = "ctb",
             email = "sam@crawley.nz",
             comment = c(ORCID = "0000-0002-7847-0411")))
Maintainer: Dominique Makowski <dom.makowski@gmail.com>
Description: Provides utilities to describe posterior
    distributions and Bayesian models. It includes point-estimates such as
    Maximum A Posteriori (MAP), measures of dispersion (Highest Density
    Interval - HDI; Kruschke, 2015 <doi:10.1016/C2012-0-00477-2>) and
    indices used for null-hypothesis testing (such as ROPE percentage, pd
    and Bayes factors). References: Makowski et al. (2021) <doi:10.21105/joss.01541>.
Depends: R (>= 3.6)
Imports: insight (>= 0.19.8), datawizard (>= 0.9.1), graphics, methods,
        stats, utils
Suggests: BayesFactor (>= 0.9.12-4.4), bayesQR, bayesplot, BH, blavaan,
        bridgesampling, brms, curl, effectsize, emmeans, gamm4, ggdist,
        ggplot2, glmmTMB, httr, KernSmooth, knitr, lavaan, lme4,
        logspline (>= 2.1.21), MASS, mclust, mediation, modelbased,
        parameters, patchwork, performance, quadprog, posterior,
        RcppEigen, rmarkdown, rstan, rstanarm, see (>= 0.7.5),
        testthat, tweedie
License: GPL-3
URL: https://easystats.github.io/bayestestR/
BugReports: https://github.com/easystats/bayestestR/issues
VignetteBuilder: knitr
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.1
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/rcmdcheck/ignore-inconsequential-notes: true
Config/Needs/website: rstudio/bslib, r-lib/pkgdown,
        easystats/easystatstemplate
NeedsCompilation: no
Packaged: 2024-02-11 10:28:23 UTC; domma
Author: Dominique Makowski [aut, cre] (<https://orcid.org/0000-0001-5375-9967>,
    @Dom_Makowski),
  Daniel Lüdecke [aut] (<https://orcid.org/0000-0002-8895-3206>,
    @strengejacke),
  Mattan S. Ben-Shachar [aut] (<https://orcid.org/0000-0002-4287-4801>,
    @mattansb),
  Indrajeet Patil [aut] (<https://orcid.org/0000-0003-1995-6531>,
    @patilindrajeets),
  Michael D. Wilson [aut] (<https://orcid.org/0000-0003-4143-7308>),
  Brenton M. Wiernik [aut] (<https://orcid.org/0000-0001-9560-6336>,
    @bmwiernik),
  Paul-Christian Bürkner [rev],
  Tristan Mahr [rev] (<https://orcid.org/0000-0002-8890-5116>),
  Henrik Singmann [ctb] (<https://orcid.org/0000-0002-4842-3657>),
  Quentin F. Gronau [ctb] (<https://orcid.org/0000-0001-5510-6943>),
  Sam Crawley [ctb] (<https://orcid.org/0000-0002-7847-0411>)
Repository: CRAN
Date/Publication: 2024-02-12 11:40:02 UTC
Built: R 4.2.3; ; 2024-02-12 15:54:16 UTC; unix
