# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,bayestestR_mediation)
S3method(as.data.frame,density)
S3method(as.double,bayestestRBF)
S3method(as.double,map_estimate)
S3method(as.double,p_direction)
S3method(as.double,p_map)
S3method(as.double,p_rope)
S3method(as.double,p_significance)
S3method(as.double,rope)
S3method(as.list,bayestestR_ci)
S3method(as.list,bayestestR_eti)
S3method(as.list,bayestestR_hdi)
S3method(as.list,bayestestR_si)
S3method(as.logical,bayesfactor_restricted)
S3method(as.matrix,bayestestRBF)
S3method(as.numeric,bayestestRBF)
S3method(as.numeric,map_estimate)
S3method(as.numeric,p_direction)
S3method(as.numeric,p_map)
S3method(as.numeric,p_significance)
S3method(as.vector,p_direction)
S3method(bayesfactor_inclusion,BFBayesFactor)
S3method(bayesfactor_inclusion,bayesfactor_models)
S3method(bayesfactor_models,BFBayesFactor)
S3method(bayesfactor_models,CmdStanFit)
S3method(bayesfactor_models,blavaan)
S3method(bayesfactor_models,brmsfit)
S3method(bayesfactor_models,default)
S3method(bayesfactor_models,stanfit)
S3method(bayesfactor_models,stanreg)
S3method(bayesfactor_parameters,CmdStanFit)
S3method(bayesfactor_parameters,bayesfactor_models)
S3method(bayesfactor_parameters,blavaan)
S3method(bayesfactor_parameters,brmsfit)
S3method(bayesfactor_parameters,comparisons)
S3method(bayesfactor_parameters,data.frame)
S3method(bayesfactor_parameters,draws)
S3method(bayesfactor_parameters,emmGrid)
S3method(bayesfactor_parameters,emm_list)
S3method(bayesfactor_parameters,numeric)
S3method(bayesfactor_parameters,predictions)
S3method(bayesfactor_parameters,rvar)
S3method(bayesfactor_parameters,sim)
S3method(bayesfactor_parameters,sim.merMod)
S3method(bayesfactor_parameters,slopes)
S3method(bayesfactor_parameters,stanfit)
S3method(bayesfactor_parameters,stanreg)
S3method(bayesfactor_restricted,CmdStanFit)
S3method(bayesfactor_restricted,blavaan)
S3method(bayesfactor_restricted,brmsfit)
S3method(bayesfactor_restricted,comparisons)
S3method(bayesfactor_restricted,data.frame)
S3method(bayesfactor_restricted,draws)
S3method(bayesfactor_restricted,emmGrid)
S3method(bayesfactor_restricted,emm_list)
S3method(bayesfactor_restricted,matrix)
S3method(bayesfactor_restricted,predictions)
S3method(bayesfactor_restricted,rvar)
S3method(bayesfactor_restricted,slopes)
S3method(bayesfactor_restricted,stanfit)
S3method(bayesfactor_restricted,stanreg)
S3method(bci,BFBayesFactor)
S3method(bci,BGGM)
S3method(bci,CmdStanFit)
S3method(bci,MCMCglmm)
S3method(bci,bamlss)
S3method(bci,bayesQR)
S3method(bci,bcplm)
S3method(bci,blavaan)
S3method(bci,blrm)
S3method(bci,brmsfit)
S3method(bci,comparisons)
S3method(bci,data.frame)
S3method(bci,draws)
S3method(bci,emmGrid)
S3method(bci,emm_list)
S3method(bci,get_predicted)
S3method(bci,mcmc)
S3method(bci,mcmc.list)
S3method(bci,numeric)
S3method(bci,predictions)
S3method(bci,rvar)
S3method(bci,sim)
S3method(bci,sim.merMod)
S3method(bci,slopes)
S3method(bci,stanfit)
S3method(bci,stanreg)
S3method(check_prior,blavaan)
S3method(check_prior,brmsfit)
S3method(check_prior,stanreg)
S3method(ci,BFBayesFactor)
S3method(ci,BGGM)
S3method(ci,CmdStanFit)
S3method(ci,MCMCglmm)
S3method(ci,bamlss)
S3method(ci,bcplm)
S3method(ci,blavaan)
S3method(ci,blrm)
S3method(ci,brmsfit)
S3method(ci,comparisons)
S3method(ci,data.frame)
S3method(ci,draws)
S3method(ci,emmGrid)
S3method(ci,emm_list)
S3method(ci,get_predicted)
S3method(ci,mcmc)
S3method(ci,mcmc.list)
S3method(ci,numeric)
S3method(ci,predictions)
S3method(ci,rvar)
S3method(ci,sim)
S3method(ci,sim.merMod)
S3method(ci,slopes)
S3method(ci,stanfit)
S3method(ci,stanreg)
S3method(describe_posterior,BFBayesFactor)
S3method(describe_posterior,BGGM)
S3method(describe_posterior,CmdStanFit)
S3method(describe_posterior,MCMCglmm)
S3method(describe_posterior,bamlss)
S3method(describe_posterior,bayesQR)
S3method(describe_posterior,bcplm)
S3method(describe_posterior,blavaan)
S3method(describe_posterior,blrm)
S3method(describe_posterior,brmsfit)
S3method(describe_posterior,comparisons)
S3method(describe_posterior,data.frame)
S3method(describe_posterior,default)
S3method(describe_posterior,double)
S3method(describe_posterior,draws)
S3method(describe_posterior,effectsize_std_params)
S3method(describe_posterior,emmGrid)
S3method(describe_posterior,emm_list)
S3method(describe_posterior,get_predicted)
S3method(describe_posterior,mcmc)
S3method(describe_posterior,mcmc.list)
S3method(describe_posterior,numeric)
S3method(describe_posterior,predictions)
S3method(describe_posterior,rvar)
S3method(describe_posterior,sim)
S3method(describe_posterior,sim.merMod)
S3method(describe_posterior,slopes)
S3method(describe_posterior,stanfit)
S3method(describe_posterior,stanmvreg)
S3method(describe_posterior,stanreg)
S3method(describe_prior,BFBayesFactor)
S3method(describe_prior,BGGM)
S3method(describe_prior,bamlss)
S3method(describe_prior,bcplm)
S3method(describe_prior,blavaan)
S3method(describe_prior,brmsfit)
S3method(describe_prior,draws)
S3method(describe_prior,rvar)
S3method(describe_prior,stanreg)
S3method(diagnostic_draws,brmsfit)
S3method(diagnostic_posterior,CmdStanFit)
S3method(diagnostic_posterior,blavaan)
S3method(diagnostic_posterior,brmsfit)
S3method(diagnostic_posterior,default)
S3method(diagnostic_posterior,stanfit)
S3method(diagnostic_posterior,stanmvreg)
S3method(diagnostic_posterior,stanreg)
S3method(display,bayesfactor_inclusion)
S3method(display,bayesfactor_models)
S3method(display,bayesfactor_parameters)
S3method(display,bayesfactor_restricted)
S3method(display,bayestestR_eti)
S3method(display,bayestestR_hdi)
S3method(display,bayestestR_si)
S3method(display,describe_posterior)
S3method(display,map_estimate)
S3method(display,p_direction)
S3method(display,p_map)
S3method(display,p_rope)
S3method(display,p_significance)
S3method(display,point_estimate)
S3method(effective_sample,CmdStanFit)
S3method(effective_sample,MCMCglmm)
S3method(effective_sample,blavaan)
S3method(effective_sample,brmsfit)
S3method(effective_sample,default)
S3method(effective_sample,stanfit)
S3method(effective_sample,stanmvreg)
S3method(effective_sample,stanreg)
S3method(equivalence_test,BFBayesFactor)
S3method(equivalence_test,CmdStanFit)
S3method(equivalence_test,bamlss)
S3method(equivalence_test,bayesQR)
S3method(equivalence_test,bcplm)
S3method(equivalence_test,blavaan)
S3method(equivalence_test,blrm)
S3method(equivalence_test,brmsfit)
S3method(equivalence_test,comparisons)
S3method(equivalence_test,data.frame)
S3method(equivalence_test,default)
S3method(equivalence_test,draws)
S3method(equivalence_test,emmGrid)
S3method(equivalence_test,emm_list)
S3method(equivalence_test,mcmc)
S3method(equivalence_test,mcmc.list)
S3method(equivalence_test,numeric)
S3method(equivalence_test,predictions)
S3method(equivalence_test,rvar)
S3method(equivalence_test,sim)
S3method(equivalence_test,sim.merMod)
S3method(equivalence_test,slopes)
S3method(equivalence_test,stanfit)
S3method(equivalence_test,stanreg)
S3method(estimate_density,BGGM)
S3method(estimate_density,CmdStanFit)
S3method(estimate_density,MCMCglmm)
S3method(estimate_density,bamlss)
S3method(estimate_density,bayesQR)
S3method(estimate_density,bcplm)
S3method(estimate_density,blavaan)
S3method(estimate_density,blrm)
S3method(estimate_density,brmsfit)
S3method(estimate_density,comparisons)
S3method(estimate_density,data.frame)
S3method(estimate_density,default)
S3method(estimate_density,double)
S3method(estimate_density,draws)
S3method(estimate_density,emmGrid)
S3method(estimate_density,emm_list)
S3method(estimate_density,grouped_df)
S3method(estimate_density,mcmc)
S3method(estimate_density,mcmc.list)
S3method(estimate_density,numeric)
S3method(estimate_density,predictions)
S3method(estimate_density,rvar)
S3method(estimate_density,slopes)
S3method(estimate_density,stanfit)
S3method(estimate_density,stanreg)
S3method(eti,BFBayesFactor)
S3method(eti,BGGM)
S3method(eti,CmdStanFit)
S3method(eti,MCMCglmm)
S3method(eti,bamlss)
S3method(eti,bayesQR)
S3method(eti,bcplm)
S3method(eti,blavaan)
S3method(eti,blrm)
S3method(eti,brmsfit)
S3method(eti,comparisons)
S3method(eti,data.frame)
S3method(eti,default)
S3method(eti,draws)
S3method(eti,emmGrid)
S3method(eti,emm_list)
S3method(eti,get_predicted)
S3method(eti,mcmc)
S3method(eti,mcmc.list)
S3method(eti,numeric)
S3method(eti,predictions)
S3method(eti,rvar)
S3method(eti,sim)
S3method(eti,sim.merMod)
S3method(eti,slopes)
S3method(eti,stanfit)
S3method(eti,stanreg)
S3method(format,bayesfactor_inclusion)
S3method(format,bayesfactor_models)
S3method(format,bayesfactor_parameters)
S3method(format,bayesfactor_restricted)
S3method(format,bayestestR_eti)
S3method(format,bayestestR_hdi)
S3method(format,bayestestR_si)
S3method(format,describe_posterior)
S3method(format,equivalence_test)
S3method(format,map_estimate)
S3method(format,p_direction)
S3method(format,p_map)
S3method(format,p_rope)
S3method(format,p_significance)
S3method(format,point_estimate)
S3method(format,rope)
S3method(hdi,BFBayesFactor)
S3method(hdi,BGGM)
S3method(hdi,CmdStanFit)
S3method(hdi,MCMCglmm)
S3method(hdi,bamlss)
S3method(hdi,bayesQR)
S3method(hdi,bcplm)
S3method(hdi,blavaan)
S3method(hdi,blrm)
S3method(hdi,brmsfit)
S3method(hdi,comparisons)
S3method(hdi,data.frame)
S3method(hdi,default)
S3method(hdi,draws)
S3method(hdi,emmGrid)
S3method(hdi,emm_list)
S3method(hdi,get_predicted)
S3method(hdi,mcmc)
S3method(hdi,mcmc.list)
S3method(hdi,numeric)
S3method(hdi,predictions)
S3method(hdi,rvar)
S3method(hdi,sim)
S3method(hdi,sim.merMod)
S3method(hdi,slopes)
S3method(hdi,stanfit)
S3method(hdi,stanreg)
S3method(map_estimate,BGGM)
S3method(map_estimate,CmdStanFit)
S3method(map_estimate,bamlss)
S3method(map_estimate,bayesQR)
S3method(map_estimate,bcplm)
S3method(map_estimate,blavaan)
S3method(map_estimate,blrm)
S3method(map_estimate,brmsfit)
S3method(map_estimate,comparisons)
S3method(map_estimate,data.frame)
S3method(map_estimate,draws)
S3method(map_estimate,emmGrid)
S3method(map_estimate,emm_list)
S3method(map_estimate,get_predicted)
S3method(map_estimate,mcmc)
S3method(map_estimate,mcmc.list)
S3method(map_estimate,numeric)
S3method(map_estimate,predictions)
S3method(map_estimate,rvar)
S3method(map_estimate,slopes)
S3method(map_estimate,stanfit)
S3method(map_estimate,stanreg)
S3method(mcse,CmdStanFit)
S3method(mcse,blavaan)
S3method(mcse,brmsfit)
S3method(mcse,stanfit)
S3method(mcse,stanreg)
S3method(mediation,brmsfit)
S3method(mediation,stanmvreg)
S3method(model_to_priors,brmsfit)
S3method(p_direction,BFBayesFactor)
S3method(p_direction,BGGM)
S3method(p_direction,CmdStanFit)
S3method(p_direction,MCMCglmm)
S3method(p_direction,bamlss)
S3method(p_direction,bayesQR)
S3method(p_direction,bcplm)
S3method(p_direction,blavaan)
S3method(p_direction,blrm)
S3method(p_direction,brmsfit)
S3method(p_direction,comparisons)
S3method(p_direction,data.frame)
S3method(p_direction,default)
S3method(p_direction,draws)
S3method(p_direction,emmGrid)
S3method(p_direction,emm_list)
S3method(p_direction,get_predicted)
S3method(p_direction,mcmc)
S3method(p_direction,mcmc.list)
S3method(p_direction,numeric)
S3method(p_direction,parameters_model)
S3method(p_direction,predictions)
S3method(p_direction,rvar)
S3method(p_direction,sim)
S3method(p_direction,sim.merMod)
S3method(p_direction,slopes)
S3method(p_direction,stanfit)
S3method(p_direction,stanreg)
S3method(p_map,BFBayesFactor)
S3method(p_map,BGGM)
S3method(p_map,CmdStanFit)
S3method(p_map,MCMCglmm)
S3method(p_map,bamlss)
S3method(p_map,bayesQR)
S3method(p_map,bcplm)
S3method(p_map,blavaan)
S3method(p_map,blrm)
S3method(p_map,brmsfit)
S3method(p_map,comparisons)
S3method(p_map,data.frame)
S3method(p_map,draws)
S3method(p_map,emmGrid)
S3method(p_map,emm_list)
S3method(p_map,get_predicted)
S3method(p_map,mcmc)
S3method(p_map,mcmc.list)
S3method(p_map,numeric)
S3method(p_map,predictions)
S3method(p_map,rvar)
S3method(p_map,sim)
S3method(p_map,sim.merMod)
S3method(p_map,slopes)
S3method(p_map,stanfit)
S3method(p_map,stanreg)
S3method(p_rope,BFBayesFactor)
S3method(p_rope,BGGM)
S3method(p_rope,CmdStanFit)
S3method(p_rope,MCMCglmm)
S3method(p_rope,bamlss)
S3method(p_rope,bcplm)
S3method(p_rope,blavaan)
S3method(p_rope,blrm)
S3method(p_rope,brmsfit)
S3method(p_rope,comparisons)
S3method(p_rope,data.frame)
S3method(p_rope,default)
S3method(p_rope,draws)
S3method(p_rope,emmGrid)
S3method(p_rope,emm_list)
S3method(p_rope,mcmc)
S3method(p_rope,mcmc.list)
S3method(p_rope,numeric)
S3method(p_rope,predictions)
S3method(p_rope,rvar)
S3method(p_rope,sim)
S3method(p_rope,sim.merMod)
S3method(p_rope,slopes)
S3method(p_rope,stanfit)
S3method(p_rope,stanreg)
S3method(p_significance,BFBayesFactor)
S3method(p_significance,BGGM)
S3method(p_significance,CmdStanFit)
S3method(p_significance,MCMCglmm)
S3method(p_significance,bamlss)
S3method(p_significance,bayesQR)
S3method(p_significance,bcplm)
S3method(p_significance,blavaan)
S3method(p_significance,blrm)
S3method(p_significance,brmsfit)
S3method(p_significance,comparisons)
S3method(p_significance,data.frame)
S3method(p_significance,default)
S3method(p_significance,draws)
S3method(p_significance,emmGrid)
S3method(p_significance,emm_list)
S3method(p_significance,get_predicted)
S3method(p_significance,mcmc)
S3method(p_significance,mcmc.list)
S3method(p_significance,numeric)
S3method(p_significance,parameters_simulate_model)
S3method(p_significance,predictions)
S3method(p_significance,rvar)
S3method(p_significance,slopes)
S3method(p_significance,stanfit)
S3method(p_significance,stanreg)
S3method(p_to_bf,default)
S3method(p_to_bf,numeric)
S3method(pd_to_p,data.frame)
S3method(pd_to_p,numeric)
S3method(plot,bayesfactor_models)
S3method(plot,bayesfactor_parameters)
S3method(plot,bayestestR_eti)
S3method(plot,bayestestR_hdi)
S3method(plot,bayestestR_mediation)
S3method(plot,bayestestR_si)
S3method(plot,describe_posterior)
S3method(plot,equivalence_test)
S3method(plot,estimate_density)
S3method(plot,estimate_density_df)
S3method(plot,map_estimate)
S3method(plot,overlap)
S3method(plot,p_direction)
S3method(plot,p_significance)
S3method(plot,point_estimate)
S3method(plot,rope)
S3method(point_estimate,BFBayesFactor)
S3method(point_estimate,BGGM)
S3method(point_estimate,CmdStanFit)
S3method(point_estimate,MCMCglmm)
S3method(point_estimate,bamlss)
S3method(point_estimate,bayesQR)
S3method(point_estimate,bcplm)
S3method(point_estimate,blavaan)
S3method(point_estimate,blrm)
S3method(point_estimate,brmsfit)
S3method(point_estimate,comparisons)
S3method(point_estimate,data.frame)
S3method(point_estimate,default)
S3method(point_estimate,draws)
S3method(point_estimate,emmGrid)
S3method(point_estimate,emm_list)
S3method(point_estimate,get_predicted)
S3method(point_estimate,matrix)
S3method(point_estimate,mcmc)
S3method(point_estimate,mcmc.list)
S3method(point_estimate,numeric)
S3method(point_estimate,predictions)
S3method(point_estimate,rvar)
S3method(point_estimate,sim)
S3method(point_estimate,sim.merMod)
S3method(point_estimate,slopes)
S3method(point_estimate,stanfit)
S3method(point_estimate,stanreg)
S3method(print,bayesfactor_inclusion)
S3method(print,bayesfactor_matrix)
S3method(print,bayesfactor_models)
S3method(print,bayesfactor_parameters)
S3method(print,bayesfactor_restricted)
S3method(print,bayestestR_eti)
S3method(print,bayestestR_hdi)
S3method(print,bayestestR_mediation)
S3method(print,bayestestR_si)
S3method(print,describe_posterior)
S3method(print,equivalence_test)
S3method(print,map_estimate)
S3method(print,overlap)
S3method(print,p_direction)
S3method(print,p_map)
S3method(print,p_rope)
S3method(print,p_significance)
S3method(print,p_to_pseudo_bf)
S3method(print,point_estimate)
S3method(print,rope)
S3method(print,sexit)
S3method(print_html,bayesfactor_inclusion)
S3method(print_html,bayesfactor_models)
S3method(print_html,bayesfactor_parameters)
S3method(print_html,bayesfactor_restricted)
S3method(print_html,bayestestR_eti)
S3method(print_html,bayestestR_hdi)
S3method(print_html,bayestestR_si)
S3method(print_html,describe_posterior)
S3method(print_html,map_estimate)
S3method(print_html,p_direction)
S3method(print_html,p_map)
S3method(print_html,p_rope)
S3method(print_html,p_significance)
S3method(print_html,point_estimate)
S3method(print_md,bayesfactor_inclusion)
S3method(print_md,bayesfactor_models)
S3method(print_md,bayesfactor_parameters)
S3method(print_md,bayesfactor_restricted)
S3method(print_md,bayestestR_eti)
S3method(print_md,bayestestR_hdi)
S3method(print_md,bayestestR_si)
S3method(print_md,describe_posterior)
S3method(print_md,map_estimate)
S3method(print_md,p_direction)
S3method(print_md,p_map)
S3method(print_md,p_rope)
S3method(print_md,p_significance)
S3method(print_md,point_estimate)
S3method(rope,BFBayesFactor)
S3method(rope,BGGM)
S3method(rope,CmdStanFit)
S3method(rope,MCMCglmm)
S3method(rope,bamlss)
S3method(rope,bayesQR)
S3method(rope,bcplm)
S3method(rope,blavaan)
S3method(rope,blrm)
S3method(rope,brmsfit)
S3method(rope,comparisons)
S3method(rope,data.frame)
S3method(rope,default)
S3method(rope,draws)
S3method(rope,emmGrid)
S3method(rope,emm_list)
S3method(rope,get_predicted)
S3method(rope,mcmc)
S3method(rope,mcmc.list)
S3method(rope,numeric)
S3method(rope,predictions)
S3method(rope,rvar)
S3method(rope,sim)
S3method(rope,sim.merMod)
S3method(rope,slopes)
S3method(rope,stanfit)
S3method(rope,stanreg)
S3method(rope_range,data.frame)
S3method(rope_range,default)
S3method(rope_range,mlm)
S3method(rope_range,parameters_model)
S3method(sensitivity_to_prior,default)
S3method(sensitivity_to_prior,stanreg)
S3method(sexit_thresholds,BFBayesFactor)
S3method(sexit_thresholds,MixMod)
S3method(sexit_thresholds,bayesQR)
S3method(sexit_thresholds,brmsfit)
S3method(sexit_thresholds,default)
S3method(sexit_thresholds,feis)
S3method(sexit_thresholds,felm)
S3method(sexit_thresholds,fixest)
S3method(sexit_thresholds,gee)
S3method(sexit_thresholds,geeglm)
S3method(sexit_thresholds,glm)
S3method(sexit_thresholds,glmmTMB)
S3method(sexit_thresholds,gls)
S3method(sexit_thresholds,hurdle)
S3method(sexit_thresholds,lm)
S3method(sexit_thresholds,lme)
S3method(sexit_thresholds,merMod)
S3method(sexit_thresholds,mixed)
S3method(sexit_thresholds,mlm)
S3method(sexit_thresholds,stanreg)
S3method(sexit_thresholds,wbm)
S3method(sexit_thresholds,zeroinfl)
S3method(si,blavaan)
S3method(si,brmsfit)
S3method(si,comparisons)
S3method(si,data.frame)
S3method(si,draws)
S3method(si,emmGrid)
S3method(si,emm_list)
S3method(si,get_predicted)
S3method(si,numeric)
S3method(si,predictions)
S3method(si,rvar)
S3method(si,slopes)
S3method(si,stanfit)
S3method(si,stanreg)
S3method(simulate_prior,bcplm)
S3method(simulate_prior,blavaan)
S3method(simulate_prior,brmsfit)
S3method(simulate_prior,stanreg)
S3method(spi,BFBayesFactor)
S3method(spi,BGGM)
S3method(spi,CmdStanFit)
S3method(spi,MCMCglmm)
S3method(spi,bamlss)
S3method(spi,bayesQR)
S3method(spi,bcplm)
S3method(spi,blavaan)
S3method(spi,blrm)
S3method(spi,brmsfit)
S3method(spi,comparisons)
S3method(spi,data.frame)
S3method(spi,default)
S3method(spi,draws)
S3method(spi,emmGrid)
S3method(spi,emm_list)
S3method(spi,get_predicted)
S3method(spi,mcmc)
S3method(spi,mcmc.list)
S3method(spi,numeric)
S3method(spi,predictions)
S3method(spi,rvar)
S3method(spi,sim)
S3method(spi,sim.merMod)
S3method(spi,slopes)
S3method(spi,stanfit)
S3method(spi,stanreg)
S3method(unupdate,blavaan)
S3method(unupdate,brmsfit)
S3method(unupdate,brmsfit_multiple)
S3method(unupdate,stanreg)
S3method(update,bayesfactor_models)
S3method(weighted_posteriors,BFBayesFactor)
S3method(weighted_posteriors,CmdStanFit)
S3method(weighted_posteriors,blavaan)
S3method(weighted_posteriors,brmsfit)
S3method(weighted_posteriors,data.frame)
S3method(weighted_posteriors,stanreg)
export(area_under_curve)
export(auc)
export(bayesfactor)
export(bayesfactor_inclusion)
export(bayesfactor_models)
export(bayesfactor_parameters)
export(bayesfactor_pointnull)
export(bayesfactor_restricted)
export(bayesfactor_rope)
export(bayesian_as_frequentist)
export(bcai)
export(bci)
export(bf_inclusion)
export(bf_models)
export(bf_parameters)
export(bf_pointnull)
export(bf_restricted)
export(bf_rope)
export(bic_to_bf)
export(check_prior)
export(ci)
export(contr.bayes)
export(contr.equalprior)
export(contr.equalprior_deviations)
export(contr.equalprior_pairs)
export(contr.orthonorm)
export(convert_bayesian_as_frequentist)
export(convert_p_to_pd)
export(convert_pd_to_p)
export(density_at)
export(describe_posterior)
export(describe_prior)
export(diagnostic_draws)
export(diagnostic_posterior)
export(display)
export(distribution)
export(distribution_beta)
export(distribution_binom)
export(distribution_binomial)
export(distribution_cauchy)
export(distribution_chisq)
export(distribution_chisquared)
export(distribution_custom)
export(distribution_gamma)
export(distribution_gaussian)
export(distribution_mixture_normal)
export(distribution_nbinom)
export(distribution_normal)
export(distribution_poisson)
export(distribution_student)
export(distribution_student_t)
export(distribution_t)
export(distribution_tweedie)
export(distribution_uniform)
export(effective_sample)
export(equivalence_test)
export(estimate_density)
export(eti)
export(hdi)
export(map_estimate)
export(mcse)
export(mediation)
export(model_to_priors)
export(overlap)
export(p_direction)
export(p_map)
export(p_pointnull)
export(p_rope)
export(p_significance)
export(p_to_bf)
export(p_to_pd)
export(pd)
export(pd_to_p)
export(point_estimate)
export(print_html)
export(print_md)
export(reshape_draws)
export(reshape_iterations)
export(rope)
export(rope_range)
export(sensitivity_to_prior)
export(sexit)
export(sexit_thresholds)
export(si)
export(simulate_correlation)
export(simulate_difference)
export(simulate_prior)
export(simulate_simpson)
export(simulate_ttest)
export(spi)
export(unupdate)
export(weighted_posteriors)
importFrom(insight,display)
importFrom(insight,print_html)
importFrom(insight,print_md)
