#' Moral Disgust Judgment
#'
#' A sample (simulated) dataset, used in tests and some examples.
#'
#' @author Richard D. Morey
#'
#' @docType data
#'
#' @name disgust
#'
#' @keywords data
#'
#' @format A data frame with 500 rows and 5 variables:
#' \describe{
#'   \item{score}{Score on the questionnaire, which ranges from 0 to 50 with higher scores representing harsher moral judgment}
#'   \item{condition}{one of three conditions, differing by the odor present in the room: a pleasant scent associated with cleanliness (lemon), a disgusting scent (sulfur), and a control condition in which no unusual odor is present}
#' }
#'
#' ```{r}
#' data("disgust")
#' head(disgust, n = 5)
#' ````
#'
NULL
