---
title: "Overview of Vignettes"
output: 
  rmarkdown::html_vignette:
vignette: >
  %\VignetteIndexEntry{Overview of Vignettes}
  \usepackage[utf8]{inputenc}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r message=FALSE, warning=FALSE, include=FALSE}
library(knitr)
knitr::opts_chunk$set(
  echo = TRUE,
  collapse = TRUE,
  warning = FALSE,
  message = FALSE,
  comment = "#>",
  eval = TRUE
)
```

All package vignettes are available at [https://easystats.github.io/bayestestR/](https://easystats.github.io/bayestestR/).

## Function Overview

* [Function Reference](https://easystats.github.io/bayestestR/reference/index.html)

## Get Started

* [Get Started with Bayesian Analysis](https://easystats.github.io/bayestestR/articles/bayestestR.html)

## Examples

1. [Initiation to Bayesian models](https://easystats.github.io/bayestestR/articles/example1.html)
2. [Confirmation of Bayesian skills](https://easystats.github.io/bayestestR/articles/example2.html)
3. [Become a Bayesian master](https://easystats.github.io/bayestestR/articles/example3.html)

## Articles

* [Credible Intervals (CI))](https://easystats.github.io/bayestestR/articles/credible_interval.html)
* [Region of Practical Equivalence (ROPE)](https://easystats.github.io/bayestestR/articles/region_of_practical_equivalence.html)
* [Probability of Direction (pd)](https://easystats.github.io/bayestestR/articles/probability_of_direction.html)
* [Bayes Factors](https://easystats.github.io/bayestestR/articles/bayes_factors.html)

## In-Depths

* [Comparison of Point-Estimates](https://easystats.github.io/bayestestR/articles/web_only/indicesEstimationComparison.html)
* [Indices of Effect Existence and Significance in the Bayesian Framework](https://www.frontiersin.org/articles/10.3389/fpsyg.2019.02767/full)
* [Mediation Analysis using Bayesian Regression Models](https://easystats.github.io/bayestestR/articles/mediation.html)

## Guidelines

* [Reporting Guidelines](https://easystats.github.io/bayestestR/articles/guidelines.html)
