% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.select_nums}
\alias{.select_nums}
\title{select numerics columns}
\usage{
.select_nums(x)
}
\description{
select numerics columns
}
\keyword{internal}
